import { createFeatureSelector, createSelector, StoreModule } from '@ngrx/store';
import { AngularFireModule } from '@angular/fire';
import { Platform } from '@ionic/angular';
import { Facebook } from '@ionic-native/facebook/ngx';
import { GooglePlus } from '@ionic-native/google-plus/ngx';
import { auth } from 'firebase/app';
import 'firebase/auth';
import { __decorate, __metadata, __awaiter } from 'tslib';
import { Injectable, InjectionToken, NgModule, Inject, defineInjectable } from '@angular/core';
import { Actions, Effect, ofType, EffectsModule } from '@ngrx/effects';
import { AngularFireAuth, AngularFireAuthModule } from '@angular/fire/auth';
import { from, Observable, of, zip } from 'rxjs';
import { catchError, exhaustMap, map, switchMap, take, mapTo } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class User {
    /**
     * @param {?} uid
     * @param {?} displayName
     * @param {?} email
     * @param {?} phoneNumber
     * @param {?=} photoURL
     * @param {?=} emailVerified
     */
    constructor(uid, displayName, email, phoneNumber, photoURL, emailVerified) {
        this.uid = uid;
        this.displayName = displayName;
        this.email = email;
        this.phoneNumber = phoneNumber;
        this.photoURL = photoURL;
        this.emailVerified = emailVerified;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {string} */
const CustomEmailHandlerActionTypes = {
    CheckActionCode: '[Auth/CustomEmailHandler] Check Action Code',
    CheckActionCodeError: '[Auth/CustomEmailHandler] Check Action Code Error',
    CheckActionCodeSuccess: '[Auth/CustomEmailHandler] Check Action Code Success',
    RecoverEmail: '[Auth/CustomEmailHandler] Recover Email',
    RecoverEmailSuccess: '[Auth/CustomEmailHandler] Recover Email Success',
    RecoverEmailError: '[Auth/CustomEmailHandler] Recover Email Error',
    ResetPassword: '[Auth/CustomEmailHandler] Reset Password',
    ResetPasswordSuccess: '[Auth/CustomEmailHandler] Reset Password Success',
    ResetPasswordError: '[Auth/CustomEmailHandler] Reset Password Error',
    VerifyEmailAddress: '[Auth/CustomEmailHandler] Verify Email Address',
    VerifyEmailAddressSuccess: '[Auth/CustomEmailHandler] Verify Email Address Success',
    VerifyEmailAddressError: '[Auth/CustomEmailHandler] Verify Email Address Error',
    VerifyPasswordResetCode: '[Auth/CustomEmailHandler] Verify Password Reset Code',
    VerifyPasswordResetCodeError: '[Auth/CustomEmailHandler] Verify Password Reset Error',
    VerifyPasswordResetCodeSuccess: '[Auth/CustomEmailHandler] Verify Password Reset Success',
};
class CheckActionCode {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = CustomEmailHandlerActionTypes.CheckActionCode;
    }
}
class CheckActionCodeError {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = CustomEmailHandlerActionTypes.CheckActionCodeError;
    }
}
class CheckActionCodeSuccess {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = CustomEmailHandlerActionTypes.CheckActionCodeSuccess;
    }
}
class RecoverEmail {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = CustomEmailHandlerActionTypes.RecoverEmail;
    }
}
class RecoverEmailError {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = CustomEmailHandlerActionTypes.RecoverEmailError;
    }
}
class RecoverEmailSuccess {
    constructor() {
        this.type = CustomEmailHandlerActionTypes.RecoverEmailSuccess;
    }
}
class ResetPassword {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = CustomEmailHandlerActionTypes.ResetPassword;
    }
}
class ResetPasswordError {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = CustomEmailHandlerActionTypes.ResetPasswordError;
    }
}
class ResetPasswordSuccess {
    constructor() {
        this.type = CustomEmailHandlerActionTypes.ResetPasswordSuccess;
    }
}
class VerifyEmailAddress {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = CustomEmailHandlerActionTypes.VerifyEmailAddress;
    }
}
class VerifyEmailAddressError {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = CustomEmailHandlerActionTypes.VerifyEmailAddressError;
    }
}
class VerifyEmailAddressSuccess {
    constructor() {
        this.type = CustomEmailHandlerActionTypes.VerifyEmailAddressSuccess;
    }
}
class VerifyPasswordResetCode {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = CustomEmailHandlerActionTypes.VerifyPasswordResetCode;
    }
}
class VerifyPasswordResetCodeError {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = CustomEmailHandlerActionTypes.VerifyPasswordResetCodeError;
    }
}
class VerifyPasswordResetCodeSuccess {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = CustomEmailHandlerActionTypes.VerifyPasswordResetCodeSuccess;
    }
}

var customEmailHandler_actions = /*#__PURE__*/Object.freeze({
    CustomEmailHandlerActionTypes: CustomEmailHandlerActionTypes,
    CheckActionCode: CheckActionCode,
    CheckActionCodeError: CheckActionCodeError,
    CheckActionCodeSuccess: CheckActionCodeSuccess,
    RecoverEmail: RecoverEmail,
    RecoverEmailError: RecoverEmailError,
    RecoverEmailSuccess: RecoverEmailSuccess,
    ResetPassword: ResetPassword,
    ResetPasswordError: ResetPasswordError,
    ResetPasswordSuccess: ResetPasswordSuccess,
    VerifyEmailAddress: VerifyEmailAddress,
    VerifyEmailAddressError: VerifyEmailAddressError,
    VerifyEmailAddressSuccess: VerifyEmailAddressSuccess,
    VerifyPasswordResetCode: VerifyPasswordResetCode,
    VerifyPasswordResetCodeError: VerifyPasswordResetCodeError,
    VerifyPasswordResetCodeSuccess: VerifyPasswordResetCodeSuccess
});

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {string} */
const PasswordManagementActionTypes = {
    ChangePasswordRequest: '[Auth/Password] Change Password Request',
    ChangePasswordError: '[Auth/Password] Change Password Error',
    ChangePasswordSuccess: '[Auth/Password] Change Password Success',
    ResetPasswordStatus: '[Auth/Password] Reset Change Password Status',
    ResetPasswordRequest: '[Auth/Password] Reset Password Request',
    ResetPasswordRequestError: '[Auth/Password] Reset Password Request Error',
    ResetPasswordRequestSuccess: '[Auth/Password] Reset Password Request Success',
};
class ChangePasswordRequest {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = PasswordManagementActionTypes.ChangePasswordRequest;
    }
}
class ChangePasswordError {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = PasswordManagementActionTypes.ChangePasswordError;
    }
}
class ChangePasswordSuccess {
    constructor() {
        this.type = PasswordManagementActionTypes.ChangePasswordSuccess;
    }
}
class ResetPasswordStatus {
    constructor() {
        this.type = PasswordManagementActionTypes.ResetPasswordStatus;
    }
}
class ResetPasswordRequest {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = PasswordManagementActionTypes.ResetPasswordRequest;
    }
}
class ResetPasswordRequestError {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = PasswordManagementActionTypes.ResetPasswordRequestError;
    }
}
class ResetPasswordRequestSuccess {
    constructor() {
        this.type = PasswordManagementActionTypes.ResetPasswordRequestSuccess;
    }
}

var passwordManagement_actions = /*#__PURE__*/Object.freeze({
    PasswordManagementActionTypes: PasswordManagementActionTypes,
    ChangePasswordRequest: ChangePasswordRequest,
    ChangePasswordError: ChangePasswordError,
    ChangePasswordSuccess: ChangePasswordSuccess,
    ResetPasswordStatus: ResetPasswordStatus,
    ResetPasswordRequest: ResetPasswordRequest,
    ResetPasswordRequestError: ResetPasswordRequestError,
    ResetPasswordRequestSuccess: ResetPasswordRequestSuccess
});

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {string} */
const ProvidersManagementActionTypes = {
    CodeSent: '[Auth/Providers] Code Sent',
    LinkCredentialAccount: '[Auth/Providers] Link Credential Account',
    LinkFacebookAccount: '[Auth/Providers] Link Facebook Account',
    LinkGoogleAccount: '[Auth/Providers] Link Google Account',
    LinkError: '[Auth/Providers] Link Error',
    LinkSuccess: '[Auth/Providers] Link Success',
    SetProviders: '[Auth/Providers] Set Providers',
    UnlinkCredentialAccount: '[Auth/Providers] Unlink Credential Account',
    UnlinkFacebookAccount: '[Auth/Providers] Unlink Facebook Account',
    UnlinkGoogleAccount: '[Auth/Providers] Unlink Google Account',
    UnlinkError: '[Auth/Providers] Unlink Error',
    UnlinkSuccess: '[Auth/Providers] Unlink Success',
    SendPhoneNumberCode: '[Auth] Send Phone Number Code',
    UnlinkPhoneNumber: '[Auth] Unlink Phone Number',
    VerifyPhoneNumber: '[Auth] Verify Phone',
};
class LinkCredentialAccount {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = ProvidersManagementActionTypes.LinkCredentialAccount;
    }
}
class LinkFacebookAccount {
    constructor() {
        this.type = ProvidersManagementActionTypes.LinkFacebookAccount;
    }
}
class LinkGoogleAccount {
    constructor() {
        this.type = ProvidersManagementActionTypes.LinkGoogleAccount;
    }
}
class VerifyPhoneNumber {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = ProvidersManagementActionTypes.VerifyPhoneNumber;
    }
}
class LinkSuccess {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = ProvidersManagementActionTypes.LinkSuccess;
    }
}
class LinkError {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = ProvidersManagementActionTypes.LinkError;
    }
}
class UnlinkSuccess {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = ProvidersManagementActionTypes.UnlinkSuccess;
    }
}
class UnlinkError {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = ProvidersManagementActionTypes.UnlinkError;
    }
}
class SendPhoneNumberCode {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = ProvidersManagementActionTypes.SendPhoneNumberCode;
    }
}
class UnlinkFacebookAccount {
    constructor() {
        this.type = ProvidersManagementActionTypes.UnlinkFacebookAccount;
    }
}
class UnlinkGoogleAccount {
    constructor() {
        this.type = ProvidersManagementActionTypes.UnlinkGoogleAccount;
    }
}
class UnlinkCredentialAccount {
    constructor() {
        this.type = ProvidersManagementActionTypes.UnlinkCredentialAccount;
    }
}
class SetProviders {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = ProvidersManagementActionTypes.SetProviders;
    }
}
class CodeSent {
    constructor() {
        this.type = ProvidersManagementActionTypes.CodeSent;
    }
}
class UnlinkPhoneNumber {
    constructor() {
        this.type = ProvidersManagementActionTypes.UnlinkPhoneNumber;
    }
}

var providersManagement_actions = /*#__PURE__*/Object.freeze({
    ProvidersManagementActionTypes: ProvidersManagementActionTypes,
    LinkCredentialAccount: LinkCredentialAccount,
    LinkFacebookAccount: LinkFacebookAccount,
    LinkGoogleAccount: LinkGoogleAccount,
    VerifyPhoneNumber: VerifyPhoneNumber,
    LinkSuccess: LinkSuccess,
    LinkError: LinkError,
    UnlinkSuccess: UnlinkSuccess,
    UnlinkError: UnlinkError,
    SendPhoneNumberCode: SendPhoneNumberCode,
    UnlinkFacebookAccount: UnlinkFacebookAccount,
    UnlinkGoogleAccount: UnlinkGoogleAccount,
    UnlinkCredentialAccount: UnlinkCredentialAccount,
    SetProviders: SetProviders,
    CodeSent: CodeSent,
    UnlinkPhoneNumber: UnlinkPhoneNumber
});

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {string} */
const AuthActionTypes = {
    Authenticated: '[Auth] Authenticated',
    AuthError: '[Auth] Error',
    CredentialsLogin: '[Auth] Credentials Login Attempt',
    CredentialsReAuthentication: '[Auth] Credentials ReAuthentication attempt',
    CredentialsRegistration: '[Auth] Credentials Registration Attempt',
    DeleteAccount: '[Auth] Delete Account',
    DeleteAccountSuccess: '[Auth] Delete Account Success',
    DeleteAccountError: '[Auth] Delete Account Error',
    FacebookLogin: '[Auth] Facebook Login Attempt',
    FacebookReAuthentication: '[Auth] Facebook ReAuthentication Attempt',
    FacebookRegistration: '[Auth] Facebook Registration Attempt',
    GetUser: '[Auth] Get User',
    GoogleLogin: '[Auth] Google Login Attempt',
    GoogleReAuthentication: '[Auth] Google ReAuthentication Attempt',
    GoogleRegistration: '[Auth] Google Registration Attempt',
    Logout: '[Auth] Logout',
    NotAuthenticated: '[Auth] Not Authenticated',
    SendVerificationEmail: '[Auth] Send Verification Email',
    ReAuthenticationError: '[Auth] ReAuthentication Error',
    ReAuthenticationSuccess: '[Auth] ReAuthentication Success',
    RefreshToken: '[Auth] ReFresh Token',
    RegistrationSuccess: '[Auth] Registration Success',
    ResetAuthState: '[Auth] Reset State',
    VerificationEmailError: '[Auth] Verification Email Error',
    VerificationEmailSent: '[Auth] Verification Email Sent',
};
class GetUser {
    /**
     * @param {?=} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = AuthActionTypes.GetUser;
    }
}
class Authenticated {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = AuthActionTypes.Authenticated;
    }
}
class NotAuthenticated {
    /**
     * @param {?=} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = AuthActionTypes.NotAuthenticated;
    }
}
class AuthError {
    /**
     * @param {?=} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = AuthActionTypes.AuthError;
    }
}
class GoogleRegistration {
    /**
     * @param {?=} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = AuthActionTypes.GoogleRegistration;
    }
}
class FacebookRegistration {
    /**
     * @param {?=} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = AuthActionTypes.FacebookRegistration;
    }
}
class CredentialsRegistration {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = AuthActionTypes.CredentialsRegistration;
    }
}
class RegistrationSuccess {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = AuthActionTypes.RegistrationSuccess;
    }
}
class Logout {
    /**
     * @param {?=} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = AuthActionTypes.Logout;
    }
}
class GoogleLogin {
    /**
     * @param {?=} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = AuthActionTypes.GoogleLogin;
    }
}
class FacebookLogin {
    /**
     * @param {?=} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = AuthActionTypes.FacebookLogin;
    }
}
class CredentialsLogin {
    /**
     * @param {?} email
     * @param {?} password
     * @param {?=} remember
     */
    constructor(email, password, remember) {
        this.email = email;
        this.password = password;
        this.remember = remember;
        this.type = AuthActionTypes.CredentialsLogin;
    }
}
class GoogleReAuthentication {
    /**
     * @param {?=} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = AuthActionTypes.GoogleReAuthentication;
    }
}
class FacebookReAuthentication {
    /**
     * @param {?=} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = AuthActionTypes.FacebookReAuthentication;
    }
}
class CredentialsReAuthentication {
    /**
     * @param {?} email
     * @param {?} password
     * @param {?=} remember
     */
    constructor(email, password, remember) {
        this.email = email;
        this.password = password;
        this.remember = remember;
        this.type = AuthActionTypes.CredentialsReAuthentication;
    }
}
class ReAuthenticationSuccess {
    constructor() {
        this.type = AuthActionTypes.ReAuthenticationSuccess;
    }
}
class ReAuthenticationError {
    /**
     * @param {?=} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = AuthActionTypes.ReAuthenticationError;
    }
}
class SendVerificationEmail {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = AuthActionTypes.SendVerificationEmail;
    }
}
class VerificationEmailError {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = AuthActionTypes.VerificationEmailError;
    }
}
class VerificationEmailSent {
    constructor() {
        this.type = AuthActionTypes.VerificationEmailSent;
    }
}
class DeleteAccount {
    constructor() {
        this.type = AuthActionTypes.DeleteAccount;
    }
}
class DeleteAccountSuccess {
    constructor() {
        this.type = AuthActionTypes.DeleteAccountSuccess;
    }
}
class DeleteAccountError {
    /**
     * @param {?} payload
     */
    constructor(payload) {
        this.payload = payload;
        this.type = AuthActionTypes.DeleteAccountError;
    }
}
class ResetAuthState {
    constructor() {
        this.type = AuthActionTypes.ResetAuthState;
    }
}
class RefreshToken {
    constructor() {
        this.type = AuthActionTypes.RefreshToken;
    }
}

var auth_actions = /*#__PURE__*/Object.freeze({
    AuthActionTypes: AuthActionTypes,
    GetUser: GetUser,
    Authenticated: Authenticated,
    NotAuthenticated: NotAuthenticated,
    AuthError: AuthError,
    GoogleRegistration: GoogleRegistration,
    FacebookRegistration: FacebookRegistration,
    CredentialsRegistration: CredentialsRegistration,
    RegistrationSuccess: RegistrationSuccess,
    Logout: Logout,
    GoogleLogin: GoogleLogin,
    FacebookLogin: FacebookLogin,
    CredentialsLogin: CredentialsLogin,
    GoogleReAuthentication: GoogleReAuthentication,
    FacebookReAuthentication: FacebookReAuthentication,
    CredentialsReAuthentication: CredentialsReAuthentication,
    ReAuthenticationSuccess: ReAuthenticationSuccess,
    ReAuthenticationError: ReAuthenticationError,
    SendVerificationEmail: SendVerificationEmail,
    VerificationEmailError: VerificationEmailError,
    VerificationEmailSent: VerificationEmailSent,
    DeleteAccount: DeleteAccount,
    DeleteAccountSuccess: DeleteAccountSuccess,
    DeleteAccountError: DeleteAccountError,
    ResetAuthState: ResetAuthState,
    RefreshToken: RefreshToken
});

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const defaultState = {
    loggedIn: false,
    user: null,
    loading: true,
    error: null,
    success: false
};
/// Reducer function
/**
 * @param {?=} state
 * @param {?=} action
 * @return {?}
 */
function reducer(state = defaultState, action) {
    switch (action.type) {
        case AuthActionTypes.GetUser:
            return Object.assign({}, state, { loading: true, success: false });
        case AuthActionTypes.Authenticated:
        case AuthActionTypes.RegistrationSuccess:
            return Object.assign({}, state, { user: action.payload.user, loading: false, loggedIn: true, success: false });
        case AuthActionTypes.NotAuthenticated:
            return Object.assign({}, state, defaultState, { loading: false, loggedIn: false, success: false });
        case AuthActionTypes.GoogleLogin:
        case AuthActionTypes.FacebookLogin:
        case AuthActionTypes.GoogleRegistration:
        case AuthActionTypes.FacebookRegistration:
        case AuthActionTypes.CredentialsLogin:
        case AuthActionTypes.CredentialsRegistration:
        case AuthActionTypes.FacebookReAuthentication:
        case AuthActionTypes.CredentialsReAuthentication:
        case AuthActionTypes.GoogleReAuthentication:
            return Object.assign({}, state, { loading: true, success: false });
        case AuthActionTypes.ResetAuthState:
            return Object.assign({}, state, { loading: false, success: false, error: null });
        case AuthActionTypes.AuthError:
        case AuthActionTypes.ReAuthenticationError:
            return Object.assign({}, state, { loading: false, success: false, error: action.payload });
        case AuthActionTypes.ReAuthenticationSuccess:
            return Object.assign({}, state, { loading: false, error: null, success: true });
        case AuthActionTypes.Logout:
            return Object.assign({}, state, { loading: true });
        case AuthActionTypes.DeleteAccount:
            return Object.assign({}, state, { loading: true, success: true, error: null });
        case AuthActionTypes.DeleteAccountError:
            return Object.assign({}, state, { loading: false, success: false, error: action.payload });
        case AuthActionTypes.DeleteAccountSuccess:
            return Object.assign({}, state, { loading: false, loggedIn: false, success: true, error: null, user: null });
        case CustomEmailHandlerActionTypes.VerifyEmailAddressSuccess:
            return Object.assign({}, state, { user: Object.assign({}, state.user, { emailVerified: true }) });
        default:
            return state;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const initialState = (/** @type {?} */ ({
    loading: false,
    success: false,
    error: null
}));
/**
 * @param {?=} state
 * @param {?=} action
 * @return {?}
 */
function reducer$1(state = initialState, action) {
    switch (action.type) {
        case PasswordManagementActionTypes.ResetPasswordRequest:
        case PasswordManagementActionTypes.ChangePasswordRequest: {
            return Object.assign({}, state, { loading: true, success: false, error: null });
        }
        case PasswordManagementActionTypes.ResetPasswordRequestError:
        case PasswordManagementActionTypes.ChangePasswordError: {
            return Object.assign({}, state, { loading: false, success: false, error: action.payload });
        }
        case PasswordManagementActionTypes.ResetPasswordRequestSuccess:
        case PasswordManagementActionTypes.ChangePasswordSuccess: {
            return Object.assign({}, state, { loading: false, success: true, error: null });
        }
        case PasswordManagementActionTypes.ResetPasswordStatus: {
            return Object.assign({}, state, { loading: false, success: false, error: null });
        }
        default:
            return state;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const initialState$1 = (/** @type {?} */ ({
    loading: false,
    success: false,
    providers: {
        facebook: false,
        google: false,
        phone: false,
        password: false
    },
    error: null
}));
/**
 * @param {?=} state
 * @param {?=} action
 * @return {?}
 */
function reducer$2(state = initialState$1, action) {
    switch (action.type) {
        case ProvidersManagementActionTypes.SetProviders:
            return Object.assign({}, state, { providers: Object.assign({}, state.providers, action.payload) });
        case ProvidersManagementActionTypes.LinkCredentialAccount:
        case ProvidersManagementActionTypes.LinkFacebookAccount:
        case ProvidersManagementActionTypes.LinkGoogleAccount:
        case ProvidersManagementActionTypes.UnlinkCredentialAccount:
        case ProvidersManagementActionTypes.UnlinkGoogleAccount:
        case ProvidersManagementActionTypes.UnlinkFacebookAccount:
            return Object.assign({}, state, { success: false, error: null, loading: true });
        case ProvidersManagementActionTypes.LinkSuccess:
            return Object.assign({}, state, { providers: Object.assign({}, state.providers, { [action.payload.provider]: true }), success: true, error: null, loading: false });
        case ProvidersManagementActionTypes.UnlinkSuccess:
            return Object.assign({}, state, { providers: Object.assign({}, state.providers, { [action.payload.provider]: false }), success: true, error: null, loading: false });
        case ProvidersManagementActionTypes.LinkError:
        case ProvidersManagementActionTypes.UnlinkError:
            return Object.assign({}, state, { success: false, error: action.payload, loading: false });
        default:
            return state;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const initialRequestStatus = {
    loading: false,
    success: false,
    error: null
};
/** @type {?} */
const initialState$2 = (/** @type {?} */ ({
    emailVerified: Object.assign({}, initialRequestStatus),
    passwordResetCode: Object.assign({}, initialRequestStatus),
    resetPassword: Object.assign({}, initialRequestStatus),
    checkCode: Object.assign({}, initialRequestStatus),
    recoverEmail: Object.assign({}, initialRequestStatus)
}));
/**
 * @param {?=} state
 * @param {?=} action
 * @return {?}
 */
function reducer$3(state = initialState$2, action) {
    switch (action.type) {
        case CustomEmailHandlerActionTypes.VerifyEmailAddress: {
            return Object.assign({}, state, { emailVerified: { loading: true, success: false, error: null } });
        }
        case CustomEmailHandlerActionTypes.VerifyEmailAddressError: {
            return Object.assign({}, state, { emailVerified: { loading: false, success: false, error: action.payload } });
        }
        case CustomEmailHandlerActionTypes.VerifyEmailAddressSuccess: {
            return Object.assign({}, state, { emailVerified: { loading: false, success: true, error: null } });
        }
        case CustomEmailHandlerActionTypes.VerifyPasswordResetCode: {
            return Object.assign({}, state, { passwordResetCode: { email: null, loading: true, success: false, error: null } });
        }
        case CustomEmailHandlerActionTypes.VerifyPasswordResetCodeError: {
            return Object.assign({}, state, { passwordResetCode: { email: null, loading: false, success: false, error: action.payload } });
        }
        case CustomEmailHandlerActionTypes.VerifyPasswordResetCodeSuccess: {
            return Object.assign({}, state, { passwordResetCode: { email: action.payload.email, loading: false, success: true, error: null } });
        }
        case CustomEmailHandlerActionTypes.CheckActionCode: {
            return Object.assign({}, state, { checkCode: { loading: true, success: false, error: null } });
        }
        case CustomEmailHandlerActionTypes.CheckActionCodeError: {
            return Object.assign({}, state, { checkCode: { loading: false, success: false, error: action.payload } });
        }
        case CustomEmailHandlerActionTypes.CheckActionCodeSuccess: {
            return Object.assign({}, state, { checkCode: { loading: false, success: true, error: null } });
        }
        case CustomEmailHandlerActionTypes.RecoverEmail: {
            return Object.assign({}, state, { recoverEmail: { loading: true, success: false, error: null } });
        }
        case CustomEmailHandlerActionTypes.RecoverEmailError: {
            return Object.assign({}, state, { recoverEmail: { loading: false, success: false, error: action.payload } });
        }
        case CustomEmailHandlerActionTypes.RecoverEmailSuccess: {
            return Object.assign({}, state, { recoverEmail: { loading: false, success: true, error: null } });
        }
        case CustomEmailHandlerActionTypes.ResetPassword: {
            return Object.assign({}, state, { resetPassword: { loading: true, success: false, error: null } });
        }
        case CustomEmailHandlerActionTypes.ResetPasswordError: {
            return Object.assign({}, state, { resetPassword: { loading: false, success: false, error: action.payload } });
        }
        case CustomEmailHandlerActionTypes.ResetPasswordSuccess: {
            return Object.assign({}, state, { resetPassword: { loading: false, success: true, error: null } });
        }
        default:
            return state;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const reducers = {
    user: reducer,
    changePassword: reducer$1,
    providers: reducer$2,
    customEmailHandler: reducer$3
};
/** @type {?} */
const getAuthState = createFeatureSelector('auth');
/** @type {?} */
const getUserState = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => state.user));
/** @type {?} */
const isAuthLoading = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => state.user.loading));
/** @type {?} */
const isUserLogged = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => state.user.loggedIn && state.user.user && state.user.user.emailVerified));
/** @type {?} */
const getUser = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => state.user.user));
/** @type {?} */
const getAuthError = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => state.user.error));
/** @type {?} */
const isAuthSuccess = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => state.user.success));
/** @type {?} */
const getChangePasswordStatus = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => state.changePassword));
/** @type {?} */
const getUserProvidersState = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => state.providers));
/** @type {?} */
const getUserProviders = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => state.providers.providers));
/** @type {?} */
const getProvidersRequestStatus = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => {
    return {
        loading: state.providers.loading,
        error: state.providers.error,
        success: state.providers.success
    };
}));
/** @type {?} */
const getCustomEmailHandlerRequestStatus = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => {
    return {
        emailVerified: {
            loading: state.customEmailHandler.emailVerified.loading,
            error: state.customEmailHandler.emailVerified.error,
            success: state.customEmailHandler.emailVerified.success
        }
    };
}));
/** @type {?} */
const getEmailVerifiedStatus = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => state.customEmailHandler.emailVerified));
/** @type {?} */
const getPasswordResetStatus = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => state.customEmailHandler.resetPassword));
/** @type {?} */
const getVerifyResetPasswordProcedure = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => state.customEmailHandler.passwordResetCode));
/** @type {?} */
const getEmailVerifiedError = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => state.customEmailHandler.emailVerified.error));
/** @type {?} */
const isEmailVerifiedLoading = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => state.customEmailHandler.emailVerified.loading));
/** @type {?} */
const getPasswordResetCodeError = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => state.customEmailHandler.passwordResetCode.error));
/** @type {?} */
const isPasswordResetCodeLoading = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => state.customEmailHandler.passwordResetCode.loading));
/** @type {?} */
const getPasswordResetCodeEmail = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => state.customEmailHandler.passwordResetCode.email));
/** @type {?} */
const getResetPasswordError = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => state.customEmailHandler.resetPassword.error));
/** @type {?} */
const isResetPasswordLoading = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => state.customEmailHandler.resetPassword.loading));
/** @type {?} */
const getCheckCodeError = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => state.customEmailHandler.checkCode.error));
/** @type {?} */
const isCheckCodeLoading = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => state.customEmailHandler.checkCode.loading));
/** @type {?} */
const getRecoverEmailError = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => state.customEmailHandler.recoverEmail.error));
/** @type {?} */
const isRecoverEmailLoading = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => state.customEmailHandler.recoverEmail.loading));
/** @type {?} */
const getResetPasswordOrCodeError = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => state.customEmailHandler.resetPassword.error || state.customEmailHandler.passwordResetCode.error));
/** @type {?} */
const getResetPasswordSuccess = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => state.customEmailHandler.resetPassword.success));
/** @type {?} */
const isResetPasswordOrCodeLoading = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => state.customEmailHandler.resetPassword.loading || state.customEmailHandler.passwordResetCode.loading));
/** @type {?} */
const getRecoverEmailOrCheckCodeError = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => state.customEmailHandler.recoverEmail.error || state.customEmailHandler.checkCode.error));
/** @type {?} */
const isRecoverEmailOrCheckCodeLoading = createSelector(getAuthState, (/**
 * @param {?} state
 * @return {?}
 */
state => state.customEmailHandler.recoverEmail.loading || state.customEmailHandler.checkCode.loading));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FirebaseNgrxUserManagementService {
    constructor() { }
}
FirebaseNgrxUserManagementService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
FirebaseNgrxUserManagementService.ctorParameters = () => [];
/** @nocollapse */ FirebaseNgrxUserManagementService.ngInjectableDef = defineInjectable({ factory: function FirebaseNgrxUserManagementService_Factory() { return new FirebaseNgrxUserManagementService(); }, token: FirebaseNgrxUserManagementService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const FIREBASE_USER_MANAGEMENT_CONFIG = new InjectionToken('userManagement.config');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const PROVIDERS_MAP = {};
PROVIDERS_MAP[auth.FacebookAuthProvider.FACEBOOK_SIGN_IN_METHOD] = 'facebook';
PROVIDERS_MAP[auth.GoogleAuthProvider.GOOGLE_SIGN_IN_METHOD] = 'google';
PROVIDERS_MAP[auth.EmailAuthProvider.EMAIL_PASSWORD_SIGN_IN_METHOD] = 'password';
PROVIDERS_MAP[auth.PhoneAuthProvider.PHONE_SIGN_IN_METHOD] = 'phone';
class LoginEffects {
    /**
     * @param {?} actions$
     * @param {?} afAuth
     * @param {?} fb
     * @param {?} gplus
     * @param {?} platform
     * @param {?} config
     */
    constructor(actions$, afAuth, fb, gplus, platform, config) {
        this.actions$ = actions$;
        this.afAuth = afAuth;
        this.fb = fb;
        this.gplus = gplus;
        this.platform = platform;
        this.config = config;
        this.getUser = this.actions$.pipe(ofType(AuthActionTypes.GetUser), exhaustMap((/**
         * @return {?}
         */
        () => this.afAuth.authState.pipe(take(1), switchMap((/**
         * @param {?} authData
         * @return {?}
         */
        authData => {
            console.debug(authData);
            if (authData) {
                /// User logged in
                console.debug('USER', authData);
                return zip(from(authData.getIdToken(true))).pipe(switchMap((/**
                 * @param {?} res
                 * @return {?}
                 */
                res => {
                    console.debug('providers found', authData.providerData);
                    /** @type {?} */
                    const providers = authData.providerData.reduce((/**
                     * @param {?} prev
                     * @param {?} current
                     * @return {?}
                     */
                    (prev, current) => {
                        /** @type {?} */
                        const key = PROVIDERS_MAP[current.providerId];
                        if (key) {
                            prev[key] = true;
                        }
                        return prev;
                    }), {});
                    console.debug(providers, authData.providerData.map((/**
                     * @param {?} p
                     * @return {?}
                     */
                    p => p.providerId)));
                    /** @type {?} */
                    const user = new User(authData.uid, authData.displayName, authData.email, authData.phoneNumber, authData.photoURL, authData.emailVerified);
                    return (/** @type {?} */ (from([new SetProviders(providers), new Authenticated({ user })])));
                })));
            }
            else {
                return (/** @type {?} */ (of(new NotAuthenticated())));
            }
        }))))));
        this.googleLogin = this.actions$.pipe(ofType(AuthActionTypes.GoogleLogin), map((/**
         * @param {?} action
         * @return {?}
         */
        (action) => action.payload)), exhaustMap((/**
         * @param {?} payload
         * @return {?}
         */
        payload => {
            return from(this.doGoogleLogin()).pipe(map((/**
             * @param {?} credential
             * @return {?}
             */
            credential => {
                // successful login
                return new GetUser();
            })), catchError((/**
             * @param {?} error
             * @return {?}
             */
            error => of(new AuthError(error)))));
        })));
        this.facebookLogin = this.actions$.pipe(ofType(AuthActionTypes.FacebookLogin), map((/**
         * @param {?} action
         * @return {?}
         */
        (action) => action.payload)), exhaustMap((/**
         * @param {?} payload
         * @return {?}
         */
        payload => {
            return from(this.doFacebookLogin()).pipe(map((/**
             * @param {?} credential
             * @return {?}
             */
            credential => {
                // successful login
                return new GetUser();
            })), catchError((/**
             * @param {?} error
             * @return {?}
             */
            error => of(new AuthError(error)))));
        })));
        this.loginWithCredentials = this.actions$.pipe(ofType(AuthActionTypes.CredentialsLogin), map((/**
         * @param {?} action
         * @return {?}
         */
        (action) => {
            return {
                email: action.email,
                password: action.password,
                remember: (action.remember) ? action.remember : false
            };
        })), exhaustMap((/**
         * @param {?} credentials
         * @return {?}
         */
        credentials => {
            return from(this.doLoginWithCredentials(credentials)).pipe(map((/**
             * @param {?} p
             * @return {?}
             */
            p => {
                // successful login
                return new GetUser();
            })), catchError((/**
             * @param {?} error
             * @return {?}
             */
            error => of(new AuthError(error)))));
        })));
        this.logout = this.actions$.pipe(ofType(AuthActionTypes.Logout), map((/**
         * @param {?} action
         * @return {?}
         */
        (action) => action.payload)), exhaustMap((/**
         * @param {?} payload
         * @return {?}
         */
        payload => {
            return from(this.afAuth.auth.signOut());
        })), map((/**
         * @param {?} authData
         * @return {?}
         */
        authData => {
            return new NotAuthenticated();
        })));
        this.refreshToken$ = this.actions$.pipe(ofType(AuthActionTypes.RefreshToken), switchMap((/**
         * @param {?} action
         * @return {?}
         */
        action => of(this.afAuth.auth.currentUser.getIdToken(true)))), switchMap((/**
         * @param {?} a
         * @return {?}
         */
        a => {
            console.debug('reloading user');
            return of(this.afAuth.auth.currentUser.reload());
        })), map((/**
         * @param {?} a
         * @return {?}
         */
        a => new GetUser())));
    }
    /**
     * @private
     * @return {?}
     */
    doFacebookLogin() {
        if (this.platform.is('cordova')) {
            return this.doFacebookLoginMobile();
        }
        else {
            /** @type {?} */
            const provider = new auth.FacebookAuthProvider();
            return this.afAuth.auth.signInWithPopup(provider);
        }
    }
    /**
     * @private
     * @return {?}
     */
    doFacebookLoginMobile() {
        return this.fb.login(['public_profile', 'email'])
            .then((/**
         * @param {?} res
         * @return {?}
         */
        (res) => {
            /** @type {?} */
            const facebookCredential = auth.FacebookAuthProvider.credential(res.authResponse.accessToken);
            return auth().signInWithCredential(facebookCredential);
        }));
    }
    /**
     * @private
     * @return {?}
     */
    doGoogleLogin() {
        if (this.platform.is('cordova')) {
            return this.doGoogleLoginMobile();
        }
        else {
            /** @type {?} */
            const provider = new auth.GoogleAuthProvider();
            provider.setCustomParameters({
                prompt: 'select_account'
            });
            return this.afAuth.auth.signInWithPopup(provider);
        }
    }
    /**
     * @private
     * @return {?}
     */
    doGoogleLoginMobile() {
        console.debug('Logging in into googlw mobile');
        /** @type {?} */
        const loginP = this.gplus.login({
            webClientId: this.config.googleWebClientId,
            offline: true,
            scopes: 'profile email'
        });
        loginP.catch(console.error);
        return loginP.then((/**
         * @param {?} gplusUser
         * @return {?}
         */
        gplusUser => {
            console.debug('Logging in into googlw mobile', gplusUser);
            return this.afAuth.auth.signInWithCredential(auth.GoogleAuthProvider.credential(gplusUser.idToken));
        }));
    }
    /**
     * @private
     * @param {?} credentials
     * @return {?}
     */
    doLoginWithCredentials(credentials) {
        if (credentials.remember) {
            return this.afAuth.auth.setPersistence(auth.Auth.Persistence.LOCAL).then((/**
             * @return {?}
             */
            () => {
                return this.afAuth.auth.signInWithEmailAndPassword(credentials.email, credentials.password);
            }));
        }
        else {
            return this.afAuth.auth.setPersistence(auth.Auth.Persistence.SESSION).then((/**
             * @return {?}
             */
            () => {
                return this.afAuth.auth.signInWithEmailAndPassword(credentials.email, credentials.password);
            }));
        }
    }
}
LoginEffects.decorators = [
    { type: Injectable }
];
/** @nocollapse */
LoginEffects.ctorParameters = () => [
    { type: Actions },
    { type: AngularFireAuth },
    { type: Facebook },
    { type: GooglePlus },
    { type: Platform },
    { type: undefined, decorators: [{ type: Inject, args: [FIREBASE_USER_MANAGEMENT_CONFIG,] }] }
];
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], LoginEffects.prototype, "getUser", void 0);
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], LoginEffects.prototype, "googleLogin", void 0);
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], LoginEffects.prototype, "facebookLogin", void 0);
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], LoginEffects.prototype, "loginWithCredentials", void 0);
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], LoginEffects.prototype, "logout", void 0);
__decorate([
    Effect(),
    __metadata("design:type", Object)
], LoginEffects.prototype, "refreshToken$", void 0);

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class RegistrationEffects {
    /**
     * @param {?} actions
     * @param {?} fb
     * @param {?} gplus
     * @param {?} platform
     * @param {?} afAuth
     * @param {?} config
     */
    constructor(actions, fb, gplus, platform, afAuth, config) {
        this.actions = actions;
        this.fb = fb;
        this.gplus = gplus;
        this.platform = platform;
        this.afAuth = afAuth;
        this.config = config;
        this.googleSignUp = this.actions.pipe(ofType(AuthActionTypes.GoogleRegistration), map((/**
         * @param {?} action
         * @return {?}
         */
        (action) => action.payload)), exhaustMap((/**
         * @param {?} payload
         * @return {?}
         */
        payload => {
            return from(this.doGoogleLogin()).pipe(switchMap((/**
             * @param {?} credential
             * @return {?}
             */
            credential => {
                console.debug('credential', credential);
                /** @type {?} */
                const authData = credential.user;
                /** @type {?} */
                const photoUrl = authData.photoURL;
                /** @type {?} */
                const user = new User(authData.uid, authData.displayName, authData.email, authData.phoneNumber, photoUrl, authData.emailVerified);
                return from([new SetProviders({ google: true }), new RegistrationSuccess({ user })]);
            })), catchError((/**
             * @param {?} error
             * @return {?}
             */
            error => {
                console.error(error);
                return of(new AuthError(error));
            })));
        })));
        this.facebookSignUp = this.actions.pipe(ofType(AuthActionTypes.FacebookRegistration), map((/**
         * @param {?} action
         * @return {?}
         */
        (action) => action.payload)), exhaustMap((/**
         * @param {?} payload
         * @return {?}
         */
        payload => {
            return from(this.doFacebookLogin()).pipe(switchMap((/**
             * @param {?} credential
             * @return {?}
             */
            credential => {
                console.debug('facebookSignUp', credential);
                /** @type {?} */
                const authData = credential.user;
                /** @type {?} */
                const photoUrl = authData.photoURL;
                console.log('PhotoUrl', photoUrl);
                /** @type {?} */
                const user = new User(authData.uid, authData.displayName, authData.email, authData.phoneNumber, photoUrl, authData.emailVerified);
                return from([new SetProviders({ facebook: true }), new RegistrationSuccess({ user })]);
            })), catchError((/**
             * @param {?} error
             * @return {?}
             */
            error => of(new AuthError(error)))));
        })));
        this.signUpWithCredentials = this.actions.pipe(ofType(AuthActionTypes.CredentialsRegistration), map((/**
         * @param {?} action
         * @return {?}
         */
        (action) => {
            return {
                email: action.payload.email,
                password: action.payload.password
            };
        })), exhaustMap((/**
         * @param {?} credentials
         * @return {?}
         */
        credentials => {
            return from(this.doSignUpWithCredentials(credentials)).pipe(map((/**
             * @param {?} credential
             * @return {?}
             */
            credential => {
                console.debug('doSignUpWithCredentials', credential);
                /** @type {?} */
                const authData = credential.user;
                /** @type {?} */
                const user = new User(authData.uid, authData.displayName, authData.email, authData.phoneNumber, authData.photoURL, authData.emailVerified);
                return new RegistrationSuccess({ user });
            })), catchError((/**
             * @param {?} error
             * @return {?}
             */
            error => of(new AuthError(error)))));
        })));
        this.sendVerificationEmail$ = this.actions.pipe(ofType(AuthActionTypes.SendVerificationEmail), map((/**
         * @param {?} action
         * @return {?}
         */
        action => action.payload)), exhaustMap((/**
         * @param {?} payload
         * @return {?}
         */
        payload => {
            return from(this.afAuth.auth.currentUser.sendEmailVerification({ url: payload.redirectUrl })).pipe(map((/**
             * @param {?} p
             * @return {?}
             */
            p => {
                return new VerificationEmailSent();
            })), catchError((/**
             * @param {?} error
             * @return {?}
             */
            error => of(new VerificationEmailError(error)))));
        })));
    }
    /**
     * @private
     * @return {?}
     */
    doFacebookLogin() {
        if (this.platform.is('cordova')) {
            return this.doFacebookLoginMobile();
        }
        else {
            /** @type {?} */
            const provider = new auth.FacebookAuthProvider();
            return this.afAuth.auth.signInWithPopup(provider);
        }
    }
    /**
     * @private
     * @return {?}
     */
    doFacebookLoginMobile() {
        return this.fb.login(['public_profile', 'email', 'user_friends', 'user_birthday'])
            .then((/**
         * @param {?} res
         * @return {?}
         */
        (res) => {
            /** @type {?} */
            const facebookCredential = auth.FacebookAuthProvider.credential(res.authResponse.accessToken);
            return auth().signInAndRetrieveDataWithCredential(facebookCredential);
        }));
    }
    /**
     * @private
     * @return {?}
     */
    doGoogleLogin() {
        if (this.platform.is('cordova')) {
            return this.doGoogleLoginMobile();
        }
        else {
            /** @type {?} */
            const provider = new auth.GoogleAuthProvider();
            provider.setCustomParameters({
                prompt: 'select_account'
            });
            return this.afAuth.auth.signInWithPopup(provider);
        }
    }
    /**
     * @private
     * @return {?}
     */
    doGoogleLoginMobile() {
        console.debug('Logging in into googlw mobile');
        /** @type {?} */
        const loginP = this.gplus.login({
            webClientId: this.config.googleWebClientId,
            offline: true,
            scopes: 'profile email'
        });
        loginP.catch(console.error);
        return loginP.then((/**
         * @param {?} gplusUser
         * @return {?}
         */
        gplusUser => {
            console.debug('Logging in into googlw mobile', gplusUser);
            return this.afAuth.auth.signInAndRetrieveDataWithCredential(auth.GoogleAuthProvider.credential(gplusUser.idToken));
        }));
    }
    /**
     * @private
     * @param {?} credentials
     * @return {?}
     */
    doSignUpWithCredentials(credentials) {
        return this.afAuth.auth.createUserWithEmailAndPassword(credentials.email, credentials.password);
    }
}
RegistrationEffects.decorators = [
    { type: Injectable }
];
/** @nocollapse */
RegistrationEffects.ctorParameters = () => [
    { type: Actions },
    { type: Facebook },
    { type: GooglePlus },
    { type: Platform },
    { type: AngularFireAuth },
    { type: undefined, decorators: [{ type: Inject, args: [FIREBASE_USER_MANAGEMENT_CONFIG,] }] }
];
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], RegistrationEffects.prototype, "googleSignUp", void 0);
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], RegistrationEffects.prototype, "facebookSignUp", void 0);
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], RegistrationEffects.prototype, "signUpWithCredentials", void 0);
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], RegistrationEffects.prototype, "sendVerificationEmail$", void 0);

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ReAuthenticationEffects {
    /**
     * @param {?} actions
     * @param {?} afAuth
     */
    constructor(actions, afAuth) {
        this.actions = actions;
        this.afAuth = afAuth;
        this.googleReAuthentication = this.actions.pipe(ofType(AuthActionTypes.GoogleReAuthentication), map((/**
         * @param {?} action
         * @return {?}
         */
        (action) => action.payload)), exhaustMap((/**
         * @param {?} payload
         * @return {?}
         */
        payload => {
            return from(this.doGoogleReAuthentication()).pipe(map((/**
             * @param {?} credential
             * @return {?}
             */
            credential => {
                // successful login
                return new ReAuthenticationSuccess();
            })), catchError((/**
             * @param {?} error
             * @return {?}
             */
            error => of(new ReAuthenticationError(error)))));
        })));
        this.facebookReAuthentication = this.actions.pipe(ofType(AuthActionTypes.FacebookReAuthentication), map((/**
         * @param {?} action
         * @return {?}
         */
        (action) => action.payload)), exhaustMap((/**
         * @param {?} payload
         * @return {?}
         */
        payload => {
            return from(this.doFacebookReAuthentication()).pipe(map((/**
             * @param {?} credential
             * @return {?}
             */
            credential => {
                // successful login
                return new ReAuthenticationSuccess();
            })), catchError((/**
             * @param {?} error
             * @return {?}
             */
            error => of(new ReAuthenticationError(error)))));
        })));
        this.reAuthenticateWithCredentials = this.actions.pipe(ofType(AuthActionTypes.CredentialsReAuthentication), map((/**
         * @param {?} action
         * @return {?}
         */
        (action) => {
            return {
                email: action.email,
                password: action.password
            };
        })), exhaustMap((/**
         * @param {?} credentials
         * @return {?}
         */
        credentials => {
            return from(this.doReAuthenticationWithCredentials(credentials)).pipe(map((/**
             * @param {?} p
             * @return {?}
             */
            p => {
                // successful login
                return new ReAuthenticationSuccess();
            })), catchError((/**
             * @param {?} error
             * @return {?}
             */
            error => of(new ReAuthenticationError(error)))));
        })));
    }
    /**
     * @private
     * @return {?}
     */
    doFacebookReAuthentication() {
        /** @type {?} */
        const provider = new auth.FacebookAuthProvider();
        return this.afAuth.auth.currentUser.reauthenticateWithPopup(provider);
    }
    /**
     * @private
     * @return {?}
     */
    doGoogleReAuthentication() {
        /** @type {?} */
        const provider = new auth.GoogleAuthProvider();
        return this.afAuth.auth.currentUser.reauthenticateWithPopup(provider);
    }
    /**
     * @private
     * @param {?} credentials
     * @return {?}
     */
    doReAuthenticationWithCredentials(credentials) {
        /** @type {?} */
        const credential = auth.EmailAuthProvider.credential(credentials.email, credentials.password);
        return this.afAuth.auth.currentUser.reauthenticateAndRetrieveDataWithCredential(credential);
    }
}
ReAuthenticationEffects.decorators = [
    { type: Injectable }
];
/** @nocollapse */
ReAuthenticationEffects.ctorParameters = () => [
    { type: Actions },
    { type: AngularFireAuth }
];
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], ReAuthenticationEffects.prototype, "googleReAuthentication", void 0);
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], ReAuthenticationEffects.prototype, "facebookReAuthentication", void 0);
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], ReAuthenticationEffects.prototype, "reAuthenticateWithCredentials", void 0);

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var EmailAuthProvider = auth.EmailAuthProvider;
class ProvidersManagementEffects {
    /**
     * @param {?} actions$
     * @param {?} fb
     * @param {?} gplus
     * @param {?} afAuth
     * @param {?} config
     */
    constructor(actions$, fb, gplus, afAuth, config) {
        this.actions$ = actions$;
        this.fb = fb;
        this.gplus = gplus;
        this.afAuth = afAuth;
        this.config = config;
        this.linkToGoogleAccount$ = this.actions$.pipe(ofType(ProvidersManagementActionTypes.LinkGoogleAccount), exhaustMap((/**
         * @param {?} actins
         * @return {?}
         */
        actins => {
            return from(this.doLinkToGoogleProvider()).pipe(mapTo(new LinkSuccess({ provider: 'google' })), catchError((/**
             * @param {?} error
             * @return {?}
             */
            error => of(new LinkError(error)))));
        })));
        this.linkToFacebookAccount$ = this.actions$.pipe(ofType(ProvidersManagementActionTypes.LinkFacebookAccount), exhaustMap((/**
         * @param {?} actions
         * @return {?}
         */
        actions => {
            return from(this.doLinkToFacebookProvider()).pipe(mapTo(new LinkSuccess({ provider: 'facebook' })), catchError((/**
             * @param {?} error
             * @return {?}
             */
            error => of(new LinkError(error)))));
        })));
        this.linkToCredentialsAccount$ = this.actions$.pipe(ofType(ProvidersManagementActionTypes.LinkCredentialAccount), map((/**
         * @param {?} action
         * @return {?}
         */
        action => action.payload)), exhaustMap((/**
         * @param {?} payload
         * @return {?}
         */
        payload => {
            return from(this.reAuthenticate().then((/**
             * @param {?} res
             * @return {?}
             */
            (res) => __awaiter(this, void 0, void 0, function* () {
                yield this.doLinkToCredentials(this.afAuth.auth.currentUser.email, payload.password);
            })))).pipe(mapTo(new LinkSuccess({ provider: 'password' })), catchError((/**
             * @param {?} error
             * @return {?}
             */
            error => {
                if (error.code === 'auth/popup-blocked') {
                    return from(this.reAuthenticateWithRedirect().then((/**
                     * @param {?} res
                     * @return {?}
                     */
                    (res) => __awaiter(this, void 0, void 0, function* () {
                        yield this.doLinkToCredentials(this.afAuth.auth.currentUser.email, payload.password);
                    })))).pipe(mapTo(new LinkSuccess({ provider: 'password' })), catchError((/**
                     * @param {?} error1
                     * @return {?}
                     */
                    error1 => of(new LinkError(error1)))));
                }
                else {
                    return of(new LinkError(error));
                }
            })));
        })));
        this.unlinkGoogleAccount$ = this.actions$.pipe(ofType(ProvidersManagementActionTypes.UnlinkGoogleAccount), exhaustMap((/**
         * @param {?} actions
         * @return {?}
         */
        actions => {
            return from(this.doUnlinkToGoogleProvider()).pipe(mapTo(new UnlinkSuccess({ provider: 'google' })), catchError((/**
             * @param {?} error
             * @return {?}
             */
            error => of(new UnlinkError(error)))));
        })));
        this.unlinkFacebookAccount$ = this.actions$.pipe(ofType(ProvidersManagementActionTypes.UnlinkFacebookAccount), exhaustMap((/**
         * @param {?} actions
         * @return {?}
         */
        actions => {
            return from(this.doUnlinkToFacebookProvider()).pipe(mapTo(new UnlinkSuccess({ provider: 'facebook' })), catchError((/**
             * @param {?} error
             * @return {?}
             */
            error => of(new UnlinkError(error)))));
        })));
        this.unlinkCredentialsAccount$ = this.actions$.pipe(ofType(ProvidersManagementActionTypes.UnlinkCredentialAccount), exhaustMap((/**
         * @param {?} action
         * @return {?}
         */
        action => {
            return from(this.reAuthenticate().then((/**
             * @param {?} res
             * @return {?}
             */
            (res) => __awaiter(this, void 0, void 0, function* () {
                yield this.doUnlinkToCredentials();
            })))).pipe(mapTo(new UnlinkSuccess({ provider: 'password' })), catchError((/**
             * @param {?} error
             * @return {?}
             */
            error => of(new UnlinkError(error)))));
        })));
        this.unlinkPhone$ = this.actions$.pipe(ofType(ProvidersManagementActionTypes.UnlinkPhoneNumber), exhaustMap((/**
         * @param {?} action
         * @return {?}
         */
        action => {
            return from(this.doUnlinkPhoneNumber()).pipe(mapTo(new UnlinkSuccess({ provider: 'phone' })), catchError((/**
             * @param {?} error
             * @return {?}
             */
            error => of(new UnlinkError(error)))));
        })));
        this.sendCodeToPhone$ = this.actions$.pipe(ofType(ProvidersManagementActionTypes.SendPhoneNumberCode), map((/**
         * @param {?} action
         * @return {?}
         */
        action => action.payload)), exhaustMap((/**
         * @param {?} payload
         * @return {?}
         */
        payload => {
            return from(((/**
             * @return {?}
             */
            () => {
                return this.afAuth.auth.currentUser.linkWithPhoneNumber(payload.number, new auth.RecaptchaVerifier(payload.captchaContainerId, {
                    size: 'invisible',
                    callback: (/**
                     * @param {?} token
                     * @return {?}
                     */
                    token => {
                        console.debug('Captcha token', token);
                    })
                })).then((/**
                 * @param {?} confirmation
                 * @return {?}
                 */
                confirmation => {
                    this.phoneNumberConfirmation = confirmation;
                }));
            }))()).pipe(mapTo(new CodeSent()), catchError((/**
             * @param {?} error
             * @return {?}
             */
            error => of(new LinkError(error)))));
        })));
        this.verifyCode$ = this.actions$.pipe(ofType(ProvidersManagementActionTypes.VerifyPhoneNumber), map((/**
         * @param {?} action
         * @return {?}
         */
        action => action.payload)), exhaustMap((/**
         * @param {?} payload
         * @return {?}
         */
        payload => {
            return from(this.phoneNumberConfirmation.confirm(payload.code)).pipe(mapTo(new LinkSuccess({ provider: 'phone' })), catchError((/**
             * @param {?} error
             * @return {?}
             */
            error => of(new LinkError(error)))));
        })));
    }
    /**
     * @private
     * @return {?}
     */
    doLinkToGoogleProvider() {
        /** @type {?} */
        const loginP = this.gplus.login({
            webClientId: this.config.googleWebClientId,
            offline: true,
            scopes: 'profile email'
        });
        loginP.catch(console.error);
        return loginP.then((/**
         * @param {?} gplusUser
         * @return {?}
         */
        gplusUser => {
            console.debug('Logging in into googlw mobile', gplusUser);
            return this.afAuth.auth.currentUser.linkWithCredential(auth.GoogleAuthProvider.credential(gplusUser.idToken));
        }));
    }
    /**
     * @private
     * @return {?}
     */
    doUnlinkToGoogleProvider() {
        /** @type {?} */
        const provider = new auth.GoogleAuthProvider();
        return this.afAuth.auth.currentUser.unlink(provider.providerId);
    }
    /**
     * @private
     * @return {?}
     */
    doLinkToFacebookProvider() {
        return this.fb.login(['public_profile', 'email', 'user_friends', 'user_birthday'])
            .then((/**
         * @param {?} res
         * @return {?}
         */
        (res) => {
            /** @type {?} */
            const facebookCredential = auth.FacebookAuthProvider.credential(res.authResponse.accessToken);
            return this.afAuth.auth.currentUser.linkWithCredential(facebookCredential);
        }));
    }
    /**
     * @private
     * @return {?}
     */
    doUnlinkToFacebookProvider() {
        /** @type {?} */
        const provider = new auth.FacebookAuthProvider();
        return this.afAuth.auth.currentUser.unlink(provider.providerId);
    }
    /**
     * @private
     * @return {?}
     */
    doUnlinkPhoneNumber() {
        /** @type {?} */
        const provider = new auth.PhoneAuthProvider();
        return this.afAuth.auth.currentUser.unlink(provider.providerId);
    }
    /**
     * @private
     * @param {?} email
     * @param {?} password
     * @return {?}
     */
    doLinkToCredentials(email, password) {
        /** @type {?} */
        const credentials = EmailAuthProvider.credential(email, password);
        return this.afAuth.auth.currentUser.linkAndRetrieveDataWithCredential(credentials);
    }
    /**
     * @private
     * @return {?}
     */
    doUnlinkToCredentials() {
        return this.afAuth.auth.currentUser.unlink(EmailAuthProvider.EMAIL_PASSWORD_SIGN_IN_METHOD);
    }
    /**
     * @private
     * @return {?}
     */
    doFacebookReAuthentication() {
        /** @type {?} */
        const provider = new auth.FacebookAuthProvider();
        return this.afAuth.auth.currentUser.reauthenticateWithPopup(provider);
    }
    /**
     * @private
     * @return {?}
     */
    doFacebookReAuthenticationWithRedirect() {
        /** @type {?} */
        const provider = new auth.FacebookAuthProvider();
        return this.afAuth.auth.currentUser.reauthenticateWithRedirect(provider);
    }
    /**
     * @private
     * @return {?}
     */
    reAuthenticate() {
        return this.afAuth.auth.fetchSignInMethodsForEmail(this.afAuth.auth.currentUser.email).then((/**
         * @param {?} res
         * @return {?}
         */
        (res) => __awaiter(this, void 0, void 0, function* () {
            if (res.indexOf('facebook.com') >= 0) {
                yield this.doFacebookReAuthentication();
                return;
            }
            else {
                yield this.doGoogleReAuthentication();
                return;
            }
        })));
    }
    /**
     * @private
     * @return {?}
     */
    reAuthenticateWithRedirect() {
        return this.afAuth.auth.fetchSignInMethodsForEmail(this.afAuth.auth.currentUser.email).then((/**
         * @param {?} res
         * @return {?}
         */
        (res) => __awaiter(this, void 0, void 0, function* () {
            if (res.indexOf('facebook.com') >= 0) {
                yield this.doFacebookReAuthenticationWithRedirect();
                return;
            }
            else {
                yield this.doGoogleReAuthenticationWithRedirect();
                return;
            }
        })));
    }
    /**
     * @private
     * @return {?}
     */
    doGoogleReAuthentication() {
        /** @type {?} */
        const provider = new auth.GoogleAuthProvider();
        return this.afAuth.auth.currentUser.reauthenticateWithPopup(provider);
    }
    /**
     * @private
     * @return {?}
     */
    doGoogleReAuthenticationWithRedirect() {
        /** @type {?} */
        const provider = new auth.GoogleAuthProvider();
        return this.afAuth.auth.currentUser.reauthenticateWithPopup(provider);
    }
}
ProvidersManagementEffects.decorators = [
    { type: Injectable }
];
/** @nocollapse */
ProvidersManagementEffects.ctorParameters = () => [
    { type: Actions },
    { type: Facebook },
    { type: GooglePlus },
    { type: AngularFireAuth },
    { type: undefined, decorators: [{ type: Inject, args: [FIREBASE_USER_MANAGEMENT_CONFIG,] }] }
];
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], ProvidersManagementEffects.prototype, "linkToGoogleAccount$", void 0);
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], ProvidersManagementEffects.prototype, "linkToFacebookAccount$", void 0);
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], ProvidersManagementEffects.prototype, "linkToCredentialsAccount$", void 0);
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], ProvidersManagementEffects.prototype, "unlinkGoogleAccount$", void 0);
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], ProvidersManagementEffects.prototype, "unlinkFacebookAccount$", void 0);
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], ProvidersManagementEffects.prototype, "unlinkCredentialsAccount$", void 0);
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], ProvidersManagementEffects.prototype, "unlinkPhone$", void 0);
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], ProvidersManagementEffects.prototype, "sendCodeToPhone$", void 0);
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], ProvidersManagementEffects.prototype, "verifyCode$", void 0);

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class PasswordManagementEffects {
    /**
     * @param {?} actions$
     * @param {?} afAuth
     */
    constructor(actions$, afAuth) {
        this.actions$ = actions$;
        this.afAuth = afAuth;
        this.changePassword = this.actions$.pipe(ofType(PasswordManagementActionTypes.ChangePasswordRequest), map((/**
         * @param {?} action
         * @return {?}
         */
        action => action.payload)), exhaustMap((/**
         * @param {?} payload
         * @return {?}
         */
        payload => {
            /** @type {?} */
            const credentials = auth.EmailAuthProvider.credential(this.afAuth.auth.currentUser.email, payload.oldPassword);
            return from(this.afAuth.auth.currentUser.reauthenticateAndRetrieveDataWithCredential(credentials).then((/**
             * @param {?} res
             * @return {?}
             */
            (res) => __awaiter(this, void 0, void 0, function* () {
                yield this.afAuth.auth.currentUser.updatePassword(payload.newPassword);
                return;
            })))).pipe(mapTo(new ChangePasswordSuccess()), catchError((/**
             * @param {?} error
             * @return {?}
             */
            error => of(new ChangePasswordError(error)))));
        })));
        this.passwordForgotten$ = this.actions$.pipe(ofType(PasswordManagementActionTypes.ResetPasswordRequest), map((/**
         * @param {?} action
         * @return {?}
         */
        action => action.payload)), exhaustMap((/**
         * @param {?} payload
         * @return {?}
         */
        payload => {
            return from(this.afAuth.auth.sendPasswordResetEmail(payload.email, { url: payload.redirectUrl })).pipe(map((/**
             * @param {?} authData
             * @return {?}
             */
            authData => {
                return new ResetPasswordRequestSuccess();
            })), catchError((/**
             * @param {?} err
             * @return {?}
             */
            err => of(new ResetPasswordRequestError(err)))));
        })));
    }
}
PasswordManagementEffects.decorators = [
    { type: Injectable }
];
/** @nocollapse */
PasswordManagementEffects.ctorParameters = () => [
    { type: Actions },
    { type: AngularFireAuth }
];
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], PasswordManagementEffects.prototype, "changePassword", void 0);
__decorate([
    Effect(),
    __metadata("design:type", Object)
], PasswordManagementEffects.prototype, "passwordForgotten$", void 0);

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CustomEmailHandlerEffects {
    /**
     * @param {?} actions$
     * @param {?} afAuth
     */
    constructor(actions$, afAuth) {
        this.actions$ = actions$;
        this.afAuth = afAuth;
        this.verifyEmail = this.actions$.pipe(ofType(CustomEmailHandlerActionTypes.VerifyEmailAddress), map((/**
         * @param {?} action
         * @return {?}
         */
        action => action.payload)), switchMap((/**
         * @param {?} payload
         * @return {?}
         */
        payload => {
            return from(this.afAuth.auth.applyActionCode(payload.actionCode).then((/**
             * @param {?} res
             * @return {?}
             */
            (res) => __awaiter(this, void 0, void 0, function* () {
            })))).pipe(mapTo(new VerifyEmailAddressSuccess()), catchError((/**
             * @param {?} error
             * @return {?}
             */
            error => of(new VerifyEmailAddressError(error)))));
        })));
        this.verifyPasswordResetCode = this.actions$.pipe(ofType(CustomEmailHandlerActionTypes.VerifyPasswordResetCode), map((/**
         * @param {?} action
         * @return {?}
         */
        action => action.payload)), switchMap((/**
         * @param {?} payload
         * @return {?}
         */
        payload => {
            return from(this.afAuth.auth.verifyPasswordResetCode(payload.actionCode)).pipe(switchMap((/**
             * @param {?} email
             * @return {?}
             */
            (email) => {
                return of(new VerifyPasswordResetCodeSuccess({ email, actionCode: payload.actionCode }));
            })), catchError((/**
             * @param {?} error
             * @return {?}
             */
            error => of(new VerifyPasswordResetCodeError(error)))));
        })));
        this.resetPassword = this.actions$.pipe(ofType(CustomEmailHandlerActionTypes.ResetPassword), map((/**
         * @param {?} action
         * @return {?}
         */
        action => action.payload)), switchMap((/**
         * @param {?} payload
         * @return {?}
         */
        payload => {
            return from(this.afAuth.auth.confirmPasswordReset(payload.actionCode, payload.newPassword)).pipe(switchMap((/**
             * @return {?}
             */
            () => {
                return of(new ResetPasswordSuccess());
            })), catchError((/**
             * @param {?} error
             * @return {?}
             */
            error => of(new ResetPasswordError(error)))));
        })));
        this.checkActionCode = this.actions$.pipe(ofType(CustomEmailHandlerActionTypes.CheckActionCode), map((/**
         * @param {?} action
         * @return {?}
         */
        action => action.payload)), switchMap((/**
         * @param {?} payload
         * @return {?}
         */
        payload => {
            return from(this.afAuth.auth.checkActionCode(payload.actionCode)).pipe(map((/**
             * @param {?} info
             * @return {?}
             */
            info => info['data']['email'])), switchMap((/**
             * @param {?} restoredEmail
             * @return {?}
             */
            (restoredEmail) => {
                return of(new CheckActionCodeSuccess({ actionCode: payload.actionCode, restoredEmail }));
            })), catchError((/**
             * @param {?} error
             * @return {?}
             */
            error => of(new CheckActionCodeError(error)))));
        })));
        this.revertOldEmail = this.actions$.pipe(ofType(CustomEmailHandlerActionTypes.RecoverEmail), map((/**
         * @param {?} action
         * @return {?}
         */
        action => action.payload)), switchMap((/**
         * @param {?} payload
         * @return {?}
         */
        payload => {
            return from(this.afAuth.auth.applyActionCode(payload.actionCode)).pipe(switchMap((/**
             * @return {?}
             */
            () => {
                return of(new RecoverEmailSuccess());
            })), catchError((/**
             * @param {?} error
             * @return {?}
             */
            error => of(new RecoverEmailError(error)))));
        })));
    }
}
CustomEmailHandlerEffects.decorators = [
    { type: Injectable }
];
/** @nocollapse */
CustomEmailHandlerEffects.ctorParameters = () => [
    { type: Actions },
    { type: AngularFireAuth }
];
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], CustomEmailHandlerEffects.prototype, "verifyEmail", void 0);
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], CustomEmailHandlerEffects.prototype, "verifyPasswordResetCode", void 0);
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], CustomEmailHandlerEffects.prototype, "resetPassword", void 0);
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], CustomEmailHandlerEffects.prototype, "checkActionCode", void 0);
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], CustomEmailHandlerEffects.prototype, "revertOldEmail", void 0);

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FirebaseNgrxUserManagementModule {
    /**
     * @param {?} config
     * @return {?}
     */
    static forRoot(config) {
        return {
            ngModule: FirebaseNgrxUserManagementModule,
            providers: [
                {
                    provide: FIREBASE_USER_MANAGEMENT_CONFIG, useValue: config
                }
            ]
        };
    }
}
FirebaseNgrxUserManagementModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    AngularFireModule,
                    AngularFireAuthModule,
                    EffectsModule.forFeature([
                        CustomEmailHandlerEffects,
                        LoginEffects,
                        ReAuthenticationEffects,
                        RegistrationEffects,
                        ProvidersManagementEffects,
                        PasswordManagementEffects
                    ]),
                    StoreModule.forFeature('auth', reducers)
                ],
                declarations: [],
                exports: [],
                providers: []
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { User, getAuthState, isAuthLoading, passwordManagement_actions as PasswordManagementActions, providersManagement_actions as ProvidersManagementActions, getChangePasswordStatus, isUserLogged, getUser, getAuthError, isAuthSuccess, getUserState, getUserProvidersState, getUserProviders, getProvidersRequestStatus, getCustomEmailHandlerRequestStatus, isEmailVerifiedLoading, getEmailVerifiedError, isRecoverEmailLoading, isResetPasswordLoading, isPasswordResetCodeLoading, isCheckCodeLoading, getPasswordResetCodeError, getRecoverEmailError, getCheckCodeError, getResetPasswordError, isResetPasswordOrCodeLoading, getResetPasswordOrCodeError, getRecoverEmailOrCheckCodeError, isRecoverEmailOrCheckCodeLoading, getPasswordResetCodeEmail, getResetPasswordSuccess, auth_actions as AuthActions, customEmailHandler_actions as CustomEmailHandlerActions, getEmailVerifiedStatus, getVerifyResetPasswordProcedure, getPasswordResetStatus, FirebaseNgrxUserManagementService, FirebaseNgrxUserManagementModule, FIREBASE_USER_MANAGEMENT_CONFIG as ɵg, CustomEmailHandlerEffects as ɵd, LoginEffects as ɵe, PasswordManagementEffects as ɵk, ProvidersManagementEffects as ɵj, ReAuthenticationEffects as ɵh, RegistrationEffects as ɵi, reducer$3 as ɵm, reducers as ɵa, reducer$1 as ɵl, reducer$2 as ɵc, reducer as ɵb };

//# sourceMappingURL=xtream-ionic-firebase-ngrx-user-management.js.map