import { Action } from '@ngrx/store';
import { User } from '../models/auth.model';
import { Credentials } from '../models/credentials';
export declare enum AuthActionTypes {
    Authenticated = "[Auth] Authenticated",
    AuthError = "[Auth] Error",
    CredentialsLogin = "[Auth] Credentials Login Attempt",
    CredentialsReAuthentication = "[Auth] Credentials ReAuthentication attempt",
    CredentialsRegistration = "[Auth] Credentials Registration Attempt",
    DeleteAccount = "[Auth] Delete Account",
    DeleteAccountSuccess = "[Auth] Delete Account Success",
    DeleteAccountError = "[Auth] Delete Account Error",
    FacebookLogin = "[Auth] Facebook Login Attempt",
    FacebookReAuthentication = "[Auth] Facebook ReAuthentication Attempt",
    FacebookRegistration = "[Auth] Facebook Registration Attempt",
    GetUser = "[Auth] Get User",
    GoogleLogin = "[Auth] Google Login Attempt",
    GoogleReAuthentication = "[Auth] Google ReAuthentication Attempt",
    GoogleRegistration = "[Auth] Google Registration Attempt",
    Logout = "[Auth] Logout",
    NotAuthenticated = "[Auth] Not Authenticated",
    SendVerificationEmail = "[Auth] Send Verification Email",
    ReAuthenticationError = "[Auth] ReAuthentication Error",
    ReAuthenticationSuccess = "[Auth] ReAuthentication Success",
    RefreshToken = "[Auth] ReFresh Token",
    RegistrationSuccess = "[Auth] Registration Success",
    ResetAuthState = "[Auth] Reset State",
    VerificationEmailError = "[Auth] Verification Email Error",
    VerificationEmailSent = "[Auth] Verification Email Sent"
}
export declare class GetUser implements Action {
    payload?: any;
    readonly type = AuthActionTypes.GetUser;
    constructor(payload?: any);
}
export declare class Authenticated implements Action {
    payload: {
        user: User;
    };
    readonly type = AuthActionTypes.Authenticated;
    constructor(payload: {
        user: User;
    });
}
export declare class NotAuthenticated implements Action {
    payload?: any;
    readonly type = AuthActionTypes.NotAuthenticated;
    constructor(payload?: any);
}
export declare class AuthError implements Action {
    payload?: any;
    readonly type = AuthActionTypes.AuthError;
    constructor(payload?: any);
}
export declare class GoogleRegistration implements Action {
    payload?: any;
    readonly type = AuthActionTypes.GoogleRegistration;
    constructor(payload?: any);
}
export declare class FacebookRegistration implements Action {
    payload?: any;
    readonly type = AuthActionTypes.FacebookRegistration;
    constructor(payload?: any);
}
export declare class CredentialsRegistration implements Action {
    payload: Credentials;
    readonly type = AuthActionTypes.CredentialsRegistration;
    constructor(payload: Credentials);
}
export declare class RegistrationSuccess implements Action {
    payload: {
        user: User;
    };
    readonly type = AuthActionTypes.RegistrationSuccess;
    constructor(payload: {
        user: User;
    });
}
export declare class Logout implements Action {
    payload?: any;
    readonly type = AuthActionTypes.Logout;
    constructor(payload?: any);
}
export declare class GoogleLogin implements Action {
    payload?: any;
    readonly type = AuthActionTypes.GoogleLogin;
    constructor(payload?: any);
}
export declare class FacebookLogin implements Action {
    payload?: any;
    readonly type = AuthActionTypes.FacebookLogin;
    constructor(payload?: any);
}
export declare class CredentialsLogin implements Action {
    email: string;
    password: string;
    remember?: boolean;
    readonly type = AuthActionTypes.CredentialsLogin;
    constructor(email: string, password: string, remember?: boolean);
}
export declare class GoogleReAuthentication implements Action {
    payload?: any;
    readonly type = AuthActionTypes.GoogleReAuthentication;
    constructor(payload?: any);
}
export declare class FacebookReAuthentication implements Action {
    payload?: any;
    readonly type = AuthActionTypes.FacebookReAuthentication;
    constructor(payload?: any);
}
export declare class CredentialsReAuthentication implements Action {
    email: string;
    password: string;
    remember?: boolean;
    readonly type = AuthActionTypes.CredentialsReAuthentication;
    constructor(email: string, password: string, remember?: boolean);
}
export declare class ReAuthenticationSuccess implements Action {
    readonly type = AuthActionTypes.ReAuthenticationSuccess;
    constructor();
}
export declare class ReAuthenticationError implements Action {
    payload?: any;
    readonly type = AuthActionTypes.ReAuthenticationError;
    constructor(payload?: any);
}
export declare class SendVerificationEmail implements Action {
    payload: {
        redirectUrl: string;
    };
    readonly type = AuthActionTypes.SendVerificationEmail;
    constructor(payload: {
        redirectUrl: string;
    });
}
export declare class VerificationEmailError implements Action {
    payload: {
        code: string;
        message: string;
    };
    readonly type = AuthActionTypes.VerificationEmailError;
    constructor(payload: {
        code: string;
        message: string;
    });
}
export declare class VerificationEmailSent implements Action {
    readonly type = AuthActionTypes.VerificationEmailSent;
    constructor();
}
export declare class DeleteAccount implements Action {
    readonly type = AuthActionTypes.DeleteAccount;
    constructor();
}
export declare class DeleteAccountSuccess implements Action {
    readonly type = AuthActionTypes.DeleteAccountSuccess;
    constructor();
}
export declare class DeleteAccountError implements Action {
    payload: {
        code: string;
        message: string;
    };
    readonly type = AuthActionTypes.DeleteAccountError;
    constructor(payload: {
        code: string;
        message: string;
    });
}
export declare class ResetAuthState implements Action {
    readonly type = AuthActionTypes.ResetAuthState;
}
export declare class RefreshToken implements Action {
    readonly type = AuthActionTypes.RefreshToken;
}
export declare type AuthActionsUnion = GetUser | Authenticated | NotAuthenticated | ReAuthenticationSuccess | ReAuthenticationError | GoogleReAuthentication | FacebookReAuthentication | CredentialsReAuthentication | GoogleLogin | FacebookLogin | CredentialsLogin | AuthError | Logout | GoogleRegistration | FacebookRegistration | CredentialsRegistration | RegistrationSuccess | SendVerificationEmail | VerificationEmailSent | VerificationEmailError | DeleteAccount | DeleteAccountError | DeleteAccountSuccess | ResetAuthState | RefreshToken;
